#!/usr/bin/env python

"""
Converts notebooks to interactive HTML pages with Juniper + Binder.

Standalone usage:
  nbjuniper TARGET (+ optional flags below)

  `nbjuniper example_notebook.ipynb` converts example_notebook.ipynb into
  example_notebook.html.

Jupyter-book usage:
  nbjuniper -j TARGET (+ optional flags below)

  `nbjuniper -j docs/` recursively adds a "Juniper" button to the dropdown menu
  of all HTML files under docs/. Clicking the button will convert all code cells
  to juniper cells.

Arguments:
  TARGET                     IPython notebook to convert. If TARGET is a
                             directory, nbjuniper will render all .ipynb files
                             in that directory (non-recursively by default).

Options:
  -h --help                  Show this screen.

  -j                         switch mode from "normal" (default) to 
                             "jupyter-book". "jupyter-book" recursively converts
                             a folder full of already-built (by `jupyter-book
                             build`) html files into juniper-enabled html files.

  -f FILENAME                yaml file containing specific settings for the
                             Juniper client. See
                             https://github.com/ines/juniper for all
                             possibilities, and this project's README.md for an
                             example.

  -r TARGET                  Recursively render all .ipynb files within TARGET
                             (should be a directory)

  --no-head                  Skip writing the HTML head to the page.

  --decapitate               Write the HTML head to a separate file
                             (juniper_head.html).

  --binderhub BINDERHUB      BinderHub instance to which to connect (default
                             is https://mybinder.org).

  --repo REPO                Github repository used to build Binder Docker
                             image on the BinderHub. (default is the very
                             minimal ashtonmv/python_binder).

  --theme THEME              Default theme is monokai. Must be a theme under
                             nbjuniper/cdn/styles
"""

import os
import json
import yaml
import sys
from markdown import markdown


def collect_notebooks(directory, extension, recursive=False):
    if recursive:
        paths = []
        for root, dirs, files in os.walk(directory):
            for name in files:
                path = os.path.join(root, name)
                if os.path.isfile(path) and path.endswith(extension):
                    paths.append(path)
    else:
        paths = [os.path.join(directory, nb) for nb in
                   os.listdir(directory) if nb.endswith(extension)]
    return paths


def get_settings():

    # Defaults
    settings = {
        "url": "https://mybinder.org",
        "repo": "ashtonmv/python_binder",
        "theme": "monokai",
        "msgLoading": " ",
        "useStorage": True,
        "isolateCells": False
    }

    for i, arg in enumerate(sys.argv):
        if arg.lower() == "-f":
            with open(sys.argv[i+1]) as f:
                settings.update(yaml.safe_load(f))

        elif arg.lower() == "--binderhub":
            settings.update({"url": sys.argv[i+1]})

        elif arg.lower() == "--repo":
            settings.update({"repo": sys.argv[i+1]})

        elif arg.lower() == "--theme":
            settings.update({"theme": sys.argv[i+1]})

    return settings


def main():

    notebooks = {}
    settings = get_settings()

    css_base = "juniperpage"
    if "-j" in sys.argv or "jupyter-book" in sys.argv:
        mode = "jupyter-book"
        css_base = "juniperbook"
    else:
        mode = "normal"

    directory = None

    if len([arg for arg in sys.argv if os.path.isdir(arg)]) != 0:
        directory = [arg for arg in sys.argv if os.path.isdir(arg)][0]

    for i, arg in enumerate(sys.argv):

        if i == 1:

            if arg.lower() in ["-h", "--help"]:
                print(__doc__)
                return

            elif mode == "normal":
                
                if os.path.isfile(arg):
                    with open(arg) as f:
                        notebooks[arg] = json.load(f)

                elif directory:
                    notebooks = {}
                    recursive = "-r" in sys.argv

                    paths = collect_notebooks(
                        directory,
                        extension=".ipynb",
                        recursive=recursive
                    )

                    for path in paths:
                        with open(path) as f:
                            notebooks[path] = json.load(f)

            elif mode == "jupyter-book":
                notebooks = [
                    nb for nb in collect_notebooks(
                        directory,
                        extension=".html",
                        recursive=True
                    )
                    if "cell docutils container" in open(nb).read()
                ]

    if len(notebooks) == 0:
        raise ValueError(
            "No convertable notebooks were found at the specified location."
            + " Please try nbjuniper -h for more information."
        )

    theme = settings["theme"]

    for k, v in settings.items():
        if type(v) != bool:
            settings[k] = f"'{v}'"
        else:
            settings[k] = str(v).lower()

    juniper_json = ", ".join(
        [f"{key}: {value}" for key, value in settings.items()]
    ) 

    juniper_init = (
        "<script>function juniperInit() {"
        + "if (! $('.juniper-cell').length) {"
        + "for (var i=0; i<$('div.highlight').length; i++) {"
        + "var codeBlock = $('div.highlight')[i];"
        + "if ($(codeBlock).parent().parent().hasClass('cell_input')) {"
        + "var pre = $(codeBlock).find('pre').first();"
        + "$(pre).attr({'data-executable': true});"
        + "var copyBtn = $(codeBlock).find('.copybtn').first();"
        + "$(copyBtn).hide(); } }"
        + "$('.cell_output').hide();"
        + f"var juniper = new Juniper({{ {juniper_json} }});"
        + "startKernel(juniper);"
        + "} };</script>"
    )

    juniper_button = (
        "<script>$(document).ready( function() {"
        + "var dropdown = $(document).find('.dropdown-buttons').last();"
        + "$(\"<a class='dropdown-buttons' onclick='juniperInit()'>"
        + "<button type='button' class='btn btn-secondary topbarbtn' title=''"
        + "data-toggle='tooltip' data-placement='left'"
        + "data-original-title='Initialize Juniper'><i class='fas fa-bolt'></i>"
        + "Juniper</button></a>\").appendTo(dropdown);"
        + "});</script>"
    )

    head = [
        "<script type='text/javascript' src='https://cdn.jsdelivr.net/gh/ashtonmv/nbjuniper@latest/cdn/juniper.min.js'></script>",
        "<script type='text/javascript' src='https://cdn.jsdelivr.net/gh/ashtonmv/nbjuniper@latest/cdn/events.min.js'></script>",
        "<script type='text/javascript' src='https://cdnjs.cloudflare.com/ajax/libs/mathjax/2.7.1/MathJax.js?config=TeX-AMS-MML_HTMLorMML'></script>",
        f"<link rel='stylesheet' href='https://cdn.jsdelivr.net/gh/ashtonmv/nbjuniper@latest/cdn/styles/{css_base}.min.css'></link>",
        f"<link rel='stylesheet' href='https://cdn.jsdelivr.net/gh/ashtonmv/nbjuniper@latest/cdn/styles/{theme}.min.css'></link>",
    ]

    if mode == "normal":
        head.insert(0, "<script type='text/javascript' src='https://code.jquery.com/jquery-3.5.1.min.js'></script>")
        head.insert(4, f"<script>$(document).ready(function() {{var juniper = new Juniper({{ {juniper_json} }}); startKernel(juniper);}});</script>",
)
        for filename in notebooks:
            notebook = notebooks[filename]
            body = ["<body>"]
            body.append("<div class='juniper-notebook'>")
            for cell in notebook["cells"]:
                if cell["cell_type"] == "code":
                    body.append("<pre data-executable>")
                    body.append("".join(cell["source"]))
                    body.append("</pre>")
                else:
                    body.append(markdown("".join(cell["source"])))
            body.append("</div>")
            body.append("</body>")

            if "--no-head" not in sys.argv and "--decapitate" not in sys.argv:
                with open(filename.replace("ipynb", "html"), "w") as o:
                    o.write("<head>")
                    o.write("\n".join(head))
                    o.write("</head>")
                    o.write("\n".join(body))
            else:
                with open(filename.replace("ipynb", "html"), "w") as o:
                    o.write("\n".join(body))

        if "--no-head" not in sys.argv:
            with open("juniper_head.html", "w") as o:
                o.write("\n".join(head))

    elif mode == "jupyter-book":

        head.insert(0, "<!-- begin nbjuniper head -->")
        head.insert(4, juniper_init)
        head.append(juniper_button)
        head.append("<!-- end nbjuniper head -->")

        for filename in notebooks:
            raw_lines = open(filename).readlines()
            with open(filename, "w") as f:
                write = True
                for line in raw_lines:
                    if "begin nbjuniper head" in line:
                        write = False
                    if write:
                        if "</head>" in line:
                            f.write("\n".join(head))
                            f.write("\n")
                        f.write(line)
                    if "end nbjuniper head" in line:
                        write = True


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nbjuniper']

package_data = \
{'': ['*']}

install_requires = \
['markdown>=3.2.2,<4.0.0', 'pyyaml>=5.3.1,<6.0.0']

entry_points = \
{'console_scripts': ['nbjuniper = nbjuniper.cli:main']}

setup_kwargs = {
    'name': 'nbjuniper',
    'version': '0.2.8',
    'description': 'Convert Jupyter Notebooks into runnable HTML files with Juniper (https://github.com/ines/juniper).',
    'long_description': '# nbjuniper\nConvert Jupyter Notebooks and Jupyter-book documentation into executable code with [Juniper](https://github.com/ines/juniper) + [Binder](https://mybinder.org)\n\n<p align="center">\n<img src="screenshot.gif"/>\n</p>\n\n---\n\n## Installation\n**conda**\n```sh\nconda install -c conda-forge nbjuniper\n```\n\n**pip**\n```sh\npip install nbjuniper\n```\n\n## Standalone (ipynb -> html) usage\n```sh\nnbjuniper example_notebook.ipynb\n```\n\nThe above command will create (or clobber if it exists!) the file example_notebook.html,\nwhich can be opened as a standalone webpage or embedded in another page. \n\n## Jupyter-book (html -> html) usage\n\nTo activate Junper for all code cells in documentation that has already been built with\n`jupyter-book build`, use the jupyter-book mode (it is automatically recursive):\n\n```sh\nnbjuniper -j documentation_folder\n```\n\nThis command adds an nbjuniper header to each HTML file in your documentation_folder. It will (should)\nonly change the appearance and behavior of code cells.\n\n## Under the hood\n\n`nbjuniper` creates _Juniper_ notebooks- they are *not* quite the same as _Ipython/Jupyter_ notebooks ([what\'s the difference?](#what-nbjuniper-cannot-do)) The html\nfile(s) created by `nbjuniper` automatically link your code to a [MyBinder](https://mybinder.org) instance that\nserves as the backend for executing the code.\n\n## Defaults and how to override them\n\n### MyBinder repository\nBy default, `nbjuniper` connects your code to my extremely minimal python Binder ([ashtonmv/python_binder](https://github.com/ashtonmv/python_binder)), where only python and its native libraries are installed. If your code has any dependencies, you\'ll\nwant to connect it to your own MyBinder docker image. If you haven\'t done so, create the MyBinder image for your repo [here](https://mybinder.org) and then run\n\n```sh\nnbjuniper example_notebook.ipynb --repo github_username/binder_repo\n```\n\nWhere you\'ve replaced `github_username` with your github username and `binder_repo` with the name of the repository for which you\'ve created the MyBinder docker image.\n\n### Using other BinderHubs\nIf you have your own BinderHub or are hosting your notebook on someone else\'s hub (e.g. [GESIS](https://notebooks.gesis.org)),\nyou\'ll want to override MyBinder.org as the default server:\n\n```sh\nnbjuniper example_notebook.ipynb --binderhub https://notebooks.gesis.org --repo github_username/binder_repo\n```\n\n### Styling\nThe default style used to create Juniper notebooks is [monokai](https://monokai.pro/). The theme controls the syntax\nhighlighting in each cell as well as the cells\' general appearance.\n\nThe easiest way to switch themes is using a bundled theme (one of those listed under cdn/styles):\n\n```sh\nnbjuniper example_notebook.ipynb --theme material\n```\n\nYou can also create your own style and hardcode it in\n(see [Removing the html head](#removing-the-html-head)). Adding new themes is very easy; see the examples under cdn/styles.\nI am slowly adding more themes that are already available for CodeMirror; if you want to help me or to add your own theme\nto the nbjuniper CDN please just submit a PR!\n\n### Full control of Juniper settings\nFor those who are familiar with [Juniper](https://github.com/ines/juniper), (and if you\'re not check it out! It\'s awesome)\nyou can customize every option used to create the Juniper client like so:\n\n```sh\nnbjuniper example_notebook.ipynb -f example_config.yaml\n```\n\nwhere example_config.yaml should have the same form as the example provided in this repository:\n\n```yaml\nurl: https://binder.michael-ashton.com  # must be a binderhub with CORS enabled\nrepo: ashtonmv/conda  # your binder repo\nisolateCells: true  # Cells don\'t pass variables to one another\nuseStorage: false  # Don\'t cache the binder (will be slow)\nmsgLoading: "Loading..."  # msg to display while loading (doesn\'t go away if no stdout!)\n...etc\n```\n\nSee the [Juniper documentation](https://github.com/ines/juniper) for a full list of settings; everthing in this file will be passed\nto the Juniper instance.\n\n### Removing the html head\nIf you\'re going to embed multiple Juniper notebooks into a single page, you don\'t want to include the html head in\neach one. That would import the stylesheet and javascript resources once per notebook, which can slow down your page load\ntime and is just sloppy. To chop off the head from a Juniper notebook, use the admittedly gruesome command\n\n```sh\nnbjuniper example_notebook.ipynb --decapitate\n```\n\nThis will create two files: the typical example_notebook.html and the severed juniper_head.html. From here you\ncan either discard juniper_head.html and write your own html head, or you can embed juniper_head.html at the top\nof your page where you\'re including the notebooks so that it\'s only read in once for the whole page. To prevent writing\nthe juniper_head.html file at all, replace `--decapitate` with `--no-head`.\n\n### What nbjuniper cannot do\nAnything that requires instantaneous feedback between the page and the MyBinder server, including certain widgets\nand tab autocompletion, won\'t work. nbjuniper also intentionally doesn\'t give you a full jupyter "header"- if you\nwant the page to look just like a jupyter notebook, there\'s a good chance you should just be redirecting to\nMyBinder itself.\n',
    'author': 'Michael Ashton',
    'author_email': 'ashtonmv@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ashtonmv/nbjuniper',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
